package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.security.RolesAllowed;
import java.util.List;

@RestController
@RequestMapping("/api/admin/")
public class AdminController {

    @Autowired
    private EmployeeService employeeService;

    public static class NewEmployeeRequest {
        private String login;
        private String name;
        private String password;
        private String photo;
        private String position;

        public String getLogin() {
            return login;
        }

        public void setLogin(String login) {
            this.login = login;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }

        public String getPhoto() {
            return photo;
        }

        public void setPhoto(String photo) {
            this.photo = photo;
        }

        public String getPosition() {
            return position;
        }

        public void setPosition(String position) {
            this.position = position;
        }
    }


    public static class DeleteEmployeeRequest {
        private Long id;


        public Long getId() {
            return id;
        }

        public void setId(Long id) {
            this.id = id;
        }
    }

    public static class UpdateEmployeeRequest {
        private Long id;
        private String field;

        public Long getId() {
            return id;
        }

        public void setId(Long id) {
            this.id = id;
        }

        public String getField() {
            return field;
        }

        public void setField(String field) {
            this.field = field;
        }
    }

    /**
     * Такой же метод, как и getEmployeeInfo, только для админов по логину
     * @return
     */
    @GetMapping("/{value}/info")
    public Employee getEmployeeInfoAdmin(@PathVariable String value) {
        return employeeService.getEmployeeInfoById(Long.parseLong(value));
    }

    @DeleteMapping("/employee/delete/")
    public ResponseEntity<String> deleteEmployee(@RequestBody DeleteEmployeeRequest deleteEmployeeRequest) {
        employeeService.deleteEmployee(deleteEmployeeRequest.id);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @PutMapping("/employee/update/login/")
    public ResponseEntity<String> updateEmployeeLogin(@RequestBody UpdateEmployeeRequest updateEmployeeRequest) {

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

        employeeService.updateEmployeeLogin(updateEmployeeRequest.id, updateEmployeeRequest.field);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @PutMapping("/employee/update/avatar/")
    public ResponseEntity<String> updateEmployeeAvatar(@RequestBody UpdateEmployeeRequest updateEmployeeRequest) {
        employeeService.updateEmployeeAvatar(updateEmployeeRequest.id, updateEmployeeRequest.field);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @PutMapping("/employee/update/position/")
    public ResponseEntity<String> updatePosition(@RequestBody UpdateEmployeeRequest updateEmployeeRequest) {
        employeeService.updateEmployeePosition(updateEmployeeRequest.id, updateEmployeeRequest.field);
        return ResponseEntity.status(HttpStatus.OK).build();
    }

    @GetMapping("/employee/info/{id}/")
    public Employee getInfo(@PathVariable Long id) {
        return employeeService.getEmployeeInfoById(id);
    }

    @GetMapping("/employee/info/all/")
    public List<Employee> getEmployees() {
        return employeeService.getEmployees();
    }

    @RolesAllowed("ADMIN")
    @PostMapping("/employee/new/")
    public ResponseEntity<String> newEmployee(@RequestBody NewEmployeeRequest newEmployeeRequest) {
        Employee employee = new Employee();
        employee.setLogin(newEmployeeRequest.getLogin());
        employee.setPhoto(newEmployeeRequest.getPhoto());
        employee.setPosition(newEmployeeRequest.getPosition());
        employee.setName(newEmployeeRequest.getName());
        employee.setPassword(newEmployeeRequest.getPassword());
        return ResponseEntity.status(HttpStatus.OK).build();
    }


}
