package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.entity.Entry;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EntryRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.service.EntryService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

@RestController
public class EmployeeController {
    private final EmployeeService employeeService;
    private final CodeRepository codeRepository;
    private final EntryService entryService;
    private final EntryRepository entryRepository;

    public EmployeeController(
            EmployeeService employeeService,
            CodeRepository codeRepository,
            EntryService entryService,
            EntryRepository entryRepository) {
        this.employeeService = employeeService;
        this.codeRepository = codeRepository;
        this.entryService = entryService;
        this.entryRepository = entryRepository;
    }

// Был вырезан коллективным решением
//    /**
//     * Эндпоинт для проверки присутствия логина в емплоеееее таблице
//     *
//     * @return статус код, смотри документацию к тз
//     */
//    @GetMapping("/api/{login}/auth")
//    public ResponseEntity<String> authAttempt(@PathVariable String login) {
//        try {
//            if (employeeService.checkEmployeeExists(login)) {
//                return ResponseEntity.status(HttpStatus.OK).build(); // Логин найден ДВЕСТИ
//            }
//            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).build(); // Логин не найден 401 неавторизован
//
//        } catch (
//                Exception e) // Я понял статус "что-то пошло не так", как то, что произошла какая-то ошибка. Вообще по идее должен быть статус 500, но допустим
//        {
//            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
//        }
//    }

    /**
     * Получить информацию по емплоеееее
     *
     * @return Return employee info.
     * Удален вариант получить 401 статус,так как теперь до этого эндпоинта нельзя достучаться без авторизации
     */
    @GetMapping("/api/info")
    public Employee getEmployeeInfo() {
        String login = SecurityContextHolder.getContext().getAuthentication().getName();
        Employee employee = employeeService.getEmployeeInfo(login).get();
        return employee;
    }

    /**
     * Проверка на дверь открыть не открыть
     *
     * @param value код двери
     * @return статус код отвта
     */
    @PatchMapping("api/open")
    public ResponseEntity<String> openDoorAttempt(@RequestBody Long value) {
        try {
            String login = SecurityContextHolder.getContext().getAuthentication().getName();

            if (!employeeService.checkEmployeeExists(login)) {
                return ResponseEntity.status(HttpStatus.UNAUTHORIZED).build();
            }

            if (!codeRepository.existsByValue(value)) {
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
            }

            employeeService.updateLastVisit(login);
            Entry entry = new Entry();
            Employee employee = employeeService.getEmployeeByLogin(login);
            entry.setEmployee(employee);
            entry.setTime(LocalDateTime.now());
            entry.setPlace(codeRepository.findByValue(value));
            entryRepository.save(entry);
            return ResponseEntity.status(HttpStatus.OK).build();

        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();
        }

    }


}
